import java.lang.Math;

/*
'********************************************************************************
'   RGB-modell Rtt,grnt och btt angivna i styrka 0->1
'********************************************************************************
'   HLS-modell dubbelkonad frgrymd med ljusstyka i Vertikal led
'   H Hue motsvarar kulr angiven som vinkel i cirkulrt spektra
'       Rd=0 Gul=60 Grn=120 Cyan=180 Bl=240 Magenta=300 Rd=360
'   L Lightness ljusstyrka gr frn svart=0 till vitt=1 rena frger=0.5
'   S Saturation frgmttnad 0->1 0 grskala=0  ren frg=1
'********************************************************************************
*/


//-----------------------------------------------------------------------
//							class RGBcol
//_______________________________________________________________________
class RGBcol  {
	double	red,
			green,
			blue;

	RGBcol(double r, double g, double b) {
		red = r;
		green = g;
		blue = b;
	}
}

//-----------------------------------------------------------------------
//							class HLScol
//_______________________________________________________________________
class HLScol  {
	double	hue,
			light,
			sat;

	HLScol(double h, double l, double s) {
		hue = h;
		light = l;
		sat = s;
	}
}




//-----------------------------------------------------------------------
//							class Misc
//_______________________________________________________________________
public class Misc {

	public void set4(Double T)  {

		T= new Double(4);
	}


	//-----------------------------------------------------------------------
	//								mini
	//_______________________________________________________________________
	public static double mini(double x1, double x2)   {

		return (x1 < x2 ? x1 : x2 );
	}

	//-----------------------------------------------------------------------
	//								maxi
	//_______________________________________________________________________
	public static double maxi(double x1, double x2)   {

		return (x1 > x2 ? x1 : x2 );
	}


	//-----------------------------------------------------------------------
	//								mini4
	//_______________________________________________________________________
	public static double mini4(double x1, double x2, double x3, double x4)   {
		double	mini1,
				mini2;

		mini1 = ( x1 < x2 ? x1 : x2 );
		mini2 = ( x3 < x4 ? x3 : x4 );
		return (mini1 < mini2 ? mini1 : mini2 );
	}

	//-----------------------------------------------------------------------
	//								maxi4
	//_______________________________________________________________________
	public static double maxi4(double x1, double x2, double x3, double x4)   {
		double	maxi1,
				maxi2;

		maxi1 = ( x1 > x2 ? x1 : x2 );
		maxi2 = ( x3 > x4 ? x3 : x4 );
		return (maxi1 > maxi2 ? maxi1 : maxi2 );
	}

	//-----------------------------------------------------------------------
	//								mini4Int
	//_______________________________________________________________________
	public static int mini4Int(int x1, int x2, int x3, int x4)   {
		int	mini1,
			mini2;

		mini1 = ( x1 < x2 ? x1 : x2 );
		mini2 = ( x3 < x4 ? x3 : x4 );
		return (mini1 < mini2 ? mini1 : mini2 );
	}

	//-----------------------------------------------------------------------
	//								maxi4Int
	//_______________________________________________________________________
	public static int maxi4Int(int x1, int x2, int x3, int x4)   {
		int	maxi1,
			maxi2;

		maxi1 = ( x1 > x2 ? x1 : x2 );
		maxi2 = ( x3 > x4 ? x3 : x4 );
		return (maxi1 > maxi2 ? maxi1 : maxi2 );
	}

	//-----------------------------------------------------------------------
	//								nChr
	//_______________________________________________________________________
	public static int nChr(double x)   {
		int  len;

		if (x >= 100)
			len = 3;
		else 
			len = 4;
		if (x<0)
			len += 1;
		if ( len > String.valueOf(x).length() ) 
			len = String.valueOf(x).length() ;
		return (len);
	}



	//-----------------------------------------------------------------------
	//								pix
	//_______________________________________________________________________
	public static int pix(int pixDown, int pixUp, double mini, double maxi, double coord)  {
		int pixCoord;

		if ( Math.abs(maxi-mini) < 0.00001 ) 
			maxi = mini + 0.00001;
		pixCoord =  pixDown + 
			(int)( (coord-mini)/(maxi-mini)*(double)(pixUp-pixDown) );
		return(pixCoord);
	}


	//' ********************************************************************
	//'   Konvertering frn RGB-frgmodell till HLS-frgmodell
	//' ********************************************************************
	public static HLScol RGBtoHLS(RGBcol rgb)  {

		double small = 0.0000001;

		double	maxVal,
				minVal,
				diff,
				rDist,
				gDist,
				bDist,
				r,g,b,
				h=0,
				l=0,
				s=0;
		
		r = rgb.red;
		g = rgb.green;
		b = rgb.blue;
		maxVal = maxi4(r,g,b,b);
		minVal = mini4(r,g,b,b);
		diff = maxVal - minVal;
		l =  (maxVal + minVal)/2 ;
		if ( Math.abs(diff) < small )  {
			s = 0;
			h = 0;
		}
		else {
			if ( l<=0.5 )
				s = diff / (maxVal+minVal) ;
			else
				s = diff / (2 - maxVal - minVal) ;
		    rDist = (maxVal - r) / diff ;
			gDist = (maxVal - g) / diff ;
			bDist = (maxVal - b) / diff ;
			if (r == maxVal)  
				h =  bDist - gDist;
			else if (g == maxVal)
				h =  2 + rDist - bDist ;
			else if (b==maxVal)
				h = 4 + gDist - rDist ;
			h = h * 60 ;
			if (h < 0)
				h = h + 360;
		}
		return ( new HLScol(h,l,s) );
	}


	//' ********************************************************************
	//'   Konvertering frn HLS-frgmodell till RGB-frgmodell
	//' ********************************************************************
	public static RGBcol HLStoRGB(HLScol hls)  {
 
		double p1,p2,
			   h,l,s,	
			   r,g,b;

		h = hls.hue;
		l = hls.light;
		s = hls.sat;
		if (l<0.5)
			p2 = l * (1 + s);
		else
			p2 = l + s - l * s;
		p1 = 2 * l - p2;
		if (s == 0)  {
			r = l;
			g = l;
			b = l;
		}
		else  {
			r = RGB(p1, p2, h + 120);
			g = RGB(p1, p2, h);
			b = RGB(p1, p2, h - 120);
		}
		return (new RGBcol(r,g,b) );
	} //' HLS to RGB


	private static double RGB(double q1 ,double q2,double hue)  {

		if (hue > 360)  hue = hue - 360;
		if (hue < 0)    hue = hue + 360;
		if (hue < 60)
			return( q1 + (q2 - q1) * hue / 60 );
		else if (hue < 180)
			return (q2);
		else if (hue < 240)
			return (q1 + (q2 - q1) * (240 - hue) / 60 );
		else
			return (q1);
	}


}

