import java.awt.*;
import java.lang.Math;
import java.io.*;

public class Pendulum extends Canvas  {

	static final Color	
		colArm1 = new Color(50,100,40), 
		colArm2 = new Color(180,40,180),
		colBase = new Color(200,60,20),
		colEk   = Color.blue,
		colEp	= Color.cyan,
		colOrig = new Color(200,60,20),
		colClone= Color.blue;

	static final double
		g=9.8;				

	static int
		curWidth,
		curHeight,
		xCtr,
		yCtr,
		baseW,
		baseL,
		pixR,
		xPts[] = new int[5],
		yPts[] = new int[5],
		ra1cosPhi1,
		ra1sinPhi1,
		rb1cosPhi1,
		rb1sinPhi1,

		ra2cosPhi,
		ra2sinPhi,
		rb2cosPhi,
		rb2sinPhi;

	static double
		a1,b1,r1,mArm1,m1,m1Pos,k1,m1Big,
		a2,b2,r2,mArm2,m2,m2Pos,k2,
		Iarm1,Im1,Im1Big,Iarm2,Im2,
		constA,constA1,constA2,constB,constC,constD,constE,
		pixPerM;

	static Image
		bufImage;

	static Graphics 
		gBuf;

	boolean
		virgin;

	//-----------------------------------------------------------------------
	//								Pendulum
	//_______________________________________________________________________
	Pendulum()  {
		setStandard();
		setConstants();
		virgin = true;
	}

	//-----------------------------------------------------------------------
	//								setStandard
	//_______________________________________________________________________
	public static void setStandard()  {
		a1=0.67;
		b1=0.075;
		r1=a1/2;
		m1Pos=0.5*r1;
		mArm1=0.438;
		m1Big=0.541;
		m1=0.242;
		k1=0;

		a2=0.68;
		b2=0.07;
		r2=a2/2;
		m2Pos=0.5*r2;
		mArm2=0.565;
		m2=0.246;
		k2=0;
	}

	//-----------------------------------------------------------------------
	//								setConstants
	//_______________________________________________________________________
	public static void setConstants()  {

		Iarm1=mArm1*(a1*a1+b1*b1)/12;
		Im1=m1*m1Pos*m1Pos;
		Im1Big=m1Big*r1*r1;
		Iarm2=mArm2*(a2*a2+b2*b2)/12;
		Im2=m2*m2Pos*m2Pos;

		constA=0.5*(Iarm1+Im1+Im1Big+Iarm2+Im2+(mArm2+m2)*r1*r1);
		constA1=0.5*(Iarm1+Im1+Im1Big);
		constA2=0.5*(Iarm2+Im2+(mArm2+m2)*r1*r1);
		constB=m2*r1*m2Pos;
		constC=0.5*(Iarm2+Im2);
		constD=(m1Big*r1+m1*m1Pos-(mArm2+m2)*r1)*g;
		constE=m2*m2Pos*g;
	}


	//-----------------------------------------------------------------------
	//								paint
	//_______________________________________________________________________

	public void paint(Graphics g)  {
		int		origin = Graph.origin;

		double	phi1,
				phi2;

		origin = Graph.origin;
		if (virgin || curWidth != size().width || curHeight != size().height)  {
			curWidth = App.pendulum.size().width;
			curHeight = App.pendulum.size().height;
			pixPerM = (curWidth<curHeight ? curWidth : curHeight) / 
                      (2.1 * (r1 + r2 + b2));        
			baseW = curWidth<curHeight ? curWidth/10 : curHeight/10;
			baseL = curWidth<curHeight ? curWidth/2 : curHeight/2;
			xCtr = curWidth / 2;
			yCtr = curHeight /2;
			bufImage = App.pendulum.createImage( curWidth, curHeight );
			virgin = false;
		}
		phi1 = Graph.y[origin][1][Graph.curFrameNo];
		phi2 = Graph.y[origin][2][Graph.curFrameNo];		
		drawPendulum(Graph.y[origin][1][Graph.curFrameNo], 
					 Graph.y[origin][2][Graph.curFrameNo]);					 
	}

	//-----------------------------------------------------------------------
	//						drawPendulum
	//_______________________________________________________________________
	public static void drawPendulum(double phi1, double phi2)  {

        gBuf = bufImage.getGraphics();
		gBuf.clearRect(0,0,curWidth,curHeight);
		// draw Base
		gBuf.setColor(colBase);
		// Hor. bar
		xPts[0] = xCtr - baseL/2;		yPts[0] = curHeight - baseW/2;
		xPts[1] = xCtr - baseL/2;		yPts[1] = curHeight;
		xPts[2] = xCtr + baseL/2;		yPts[2] = curHeight ;
		xPts[3] = xCtr + baseL/2;		yPts[3] = curHeight - baseW/2;
		xPts[4] = xPts[0];		yPts[4] = yPts[0];
		gBuf.fillPolygon(xPts, yPts, 4);
		gBuf.fillOval(xCtr - baseL/2 - baseW/2, curHeight - baseW/2, baseW, baseW);
		gBuf.fillOval(xCtr + baseL/2 - baseW/2, curHeight - baseW/2, baseW, baseW);
		//Vert. bar
		xPts[0] = xCtr - baseW/2;	yPts[0] = curHeight;
		xPts[1] = xCtr + baseW/2;	yPts[1] = curHeight;
		xPts[2] = xCtr + baseW/2;	yPts[2] = yCtr;
		xPts[3] = xCtr - baseW/2;	yPts[3] = yCtr;
		xPts[4] = xPts[0];		yPts[4] = yPts[0];
		gBuf.fillPolygon(xPts, yPts, 4);
		gBuf.fillOval(xCtr - baseW/2, yCtr - baseW/2, baseW, baseW);

		drawArm1(gBuf, phi1, a1, m1Pos, b1);
		drawArm2(gBuf, phi1, a1, phi2, a2, m2Pos, b2);

		App.pendulum.getGraphics().drawImage( bufImage, 0, 0, App.pendulum );
		gBuf.dispose();
	}

	//-----------------------------------------------------------------------
	//								drawArm1
	//_______________________________________________________________________
	private static void drawArm1(Graphics g, double phi1, double arm1L, double m1Pos,double arm1W)  {

		int pixR=0,
			pixHalfW;

		ra1cosPhi1 = (int) ( pixPerM * (arm1L/2+arm1W/2) * Math.cos(phi1) );
		ra1sinPhi1 = (int) ( pixPerM * (arm1L/2+arm1W/2) * Math.sin(phi1) );
		rb1cosPhi1 = (int) ( pixPerM * (arm1W/2)* Math.cos(phi1) );
		rb1sinPhi1 = (int) ( pixPerM * (arm1W/2) * Math.sin(phi1) );
		pixHalfW = (int) (pixPerM * arm1W / 2);
		
		// arm1
		g.setColor(colArm1);
		xPts[0] = xCtr + ra1cosPhi1 + rb1sinPhi1;
		yPts[0] = yCtr - ra1sinPhi1 + rb1cosPhi1;
		xPts[1] = xCtr + ra1cosPhi1 - rb1sinPhi1;
		yPts[1] = yCtr - ra1sinPhi1 - rb1cosPhi1;
		xPts[2] = xCtr - ra1cosPhi1 - rb1sinPhi1;
		yPts[2] = yCtr + ra1sinPhi1 - rb1cosPhi1;
		xPts[3] = xCtr - ra1cosPhi1 + rb1sinPhi1;
		yPts[3] = yCtr + ra1sinPhi1 + rb1cosPhi1;
		xPts[4] = xPts[0];		
		yPts[4] = yPts[0];
		g.fillPolygon(xPts,yPts,4);
		g.fillOval(xCtr + ra1cosPhi1 - pixHalfW, 
				   yCtr - ra1sinPhi1 - pixHalfW, 2*pixHalfW, 2*pixHalfW);
		g.fillOval(xCtr - ra1cosPhi1 - pixHalfW, 
			       yCtr + ra1sinPhi1 - pixHalfW, 2*pixHalfW, 2*pixHalfW);
		// axis
		g.setColor(new Color(230,230,30));
		pixR = pixHalfW/3;
		g.fillOval(xCtr - pixR, yCtr - pixR, 2*pixR, 2*pixR);
		// m1
		g.setColor(Color.black);
		pixR = pixHalfW/2;
		g.fillOval(xCtr + (int) (pixPerM*m1Pos*Math.cos(phi1)) - pixR, 
			       yCtr - (int) (pixPerM*m1Pos*Math.sin(phi1)) - pixR, 2*pixR, 2*pixR);
		// m1Big
		g.setColor(Color.orange);
		pixR = 3*pixHalfW/4;
		g.fillOval(xCtr + ra1cosPhi1 - pixR, 
			       yCtr - ra1sinPhi1 - pixR, 2*pixR, 2*pixR);		
	}


	//-----------------------------------------------------------------------
	//								drawArm2
	//_______________________________________________________________________	
	private static void drawArm2(Graphics g, double phi1, double arm1L, 
								double phi2, double arm2L, double m2Pos, double arm2W)  {
		
		int pixR = 0,
			pixHalfW,
			x2Ctr,y2Ctr;

		pixHalfW = (int) (pixPerM * arm2W/2);
		x2Ctr = xCtr + (int) ( pixPerM * (-arm1L/2) * Math.cos(phi1) );  // arm2W delete later
		y2Ctr = yCtr - (int) ( pixPerM * (-arm1L/2) * Math.sin(phi1) );  // arm2W delete later

		ra2cosPhi = (int) ( pixPerM * (arm2L/2) * Math.cos(phi1+phi2) );
		ra2sinPhi = (int) ( pixPerM * (arm2L/2) * Math.sin(phi1+phi2) );
		rb2cosPhi = (int) ( pixPerM * (arm2W/2) * Math.cos(phi1+phi2) );
		rb2sinPhi = (int) ( pixPerM * (arm2W/2) * Math.sin(phi1+phi2) );

		// arm2
		g.setColor(colArm2);
		xPts[0] = x2Ctr + ra2cosPhi + rb2sinPhi;
		yPts[0] = y2Ctr - ra2sinPhi + rb2cosPhi;
		xPts[1] = x2Ctr + ra2cosPhi - rb2sinPhi;
		yPts[1] = y2Ctr - ra2sinPhi - rb2cosPhi;
		xPts[2] = x2Ctr - ra2cosPhi - rb2sinPhi;
		yPts[2] = y2Ctr + ra2sinPhi - rb2cosPhi;
		xPts[3] = x2Ctr - ra2cosPhi + rb2sinPhi;
		yPts[3] = y2Ctr + ra2sinPhi + rb2cosPhi;
		xPts[4] = xPts[0];		
		yPts[4] = yPts[0];
		g.fillPolygon(xPts,yPts,4);
		g.fillOval(x2Ctr + ra2cosPhi - pixHalfW, y2Ctr - ra2sinPhi - pixHalfW, 2*pixHalfW, 2*pixHalfW);
		g.fillOval(x2Ctr - ra2cosPhi - pixHalfW, y2Ctr + ra2sinPhi - pixHalfW, 2*pixHalfW, 2*pixHalfW);

		// axis
		g.setColor(new Color(230,230,40));
		pixR = pixHalfW/3;
		g.fillOval(x2Ctr - pixR, y2Ctr - pixR, 2*pixR, 2*pixR);

		// m2
		g.setColor(Color.black);
		pixR = pixHalfW/2;
		g.fillOval(x2Ctr + (int) (pixPerM*m2Pos*Math.cos(phi1+phi2)) - pixR, 
			       y2Ctr - (int) (pixPerM*m2Pos*Math.sin(phi1+phi2)) - pixR, 2*pixR, 2*pixR);
	}

}

