import java.lang.Math;
import java.awt.*;
import java.io.*;


// ------------- Point ---------------------------------------------------------------
class Point  {
	int	x,y;
}

// ----------- PolygonInfo  class with info on polygonarea on surface -------------------
class PolygonInfo  {

	boolean
				visible;
	double
				Nx,
				Ny,
				Nz,
				xMin,
				xMax,
				yMin,
				yMax,
				xCenter,
				yCenter,
				zCenter;
				// info for filling polygonArea
	int
				pixYtop,
				nScanlines,
				pixLeftEdge[],
				pixRightEdge[];
	Color				
				color;				
}

// ------ SegmentInfo class with info of circular crossection segment of torus --------
class SegmentInfo  {
	double
				xMin,
				xMax,
				yMin,
				yMax;
}

// -------- Torus ----------------------------------------------------------------------
public class Torus   {

	static Color 
		torusColor = new Color(200,70,10);

	static RGBcol
		rgb;  // = new RGBcol(0,0,0);

	static HLScol
		hls = new HLScol(0,0,0);

	static double
		hue,
		light,
		sat;
		
	static final int
		torusMarginVer=30;
	
	static final double	// radius of torus circles
		ra = 3,
		rb = 1,
		// angular polygonsize (multiple of 4)
		da = (2*Math.PI)/200,
		db = (2*Math.PI)/80,

		// view angle
		view = 30*Math.PI/180,
		// light and color 
		light_amb = 2,	// light level background
		light_dir = 7,	// light level directed lightsource
		light_max = 15,	// Normation of lightvalues L=L_amb+L_dir 
						// Lmax gives max light=white

		// normed direction to lightsource
		light_x = 0,
		light_y = 0,
		light_z = 1;


	static int
		torusMarginHor,
		nStepsDa,
		nStepsDb,
		pixMaxLeft,
		pixMaxRight,
		pixMaxUp,
		pixMaxDown;

	static double
		torusXmin,
		torusXmax,
		torusYmin,
		torusYmax,
		vcData[][][],
		rotMatrix[][] = new double[4][4];

	static SegmentInfo
		segments[];

	static PolygonInfo
		polygons[][];

	

	//-----------------------------------------------------------------------
	//								Torus
	//_______________________________________________________________________
	Torus()  {
	}

	//-----------------------------------------------------------------------
	//								getRotMatrix
	//_______________________________________________________________________

	static private void getRotMatrix()  {
		rotMatrix[1][1] = 1; 
		rotMatrix[1][2] = 0; 
		rotMatrix[1][3] = 0;

		rotMatrix[2][1] = 0;
		rotMatrix[2][2] = Math.cos(view);
		rotMatrix[2][3] = -Math.sin(view);
		
		rotMatrix[3][1] = 0; 
		rotMatrix[3][2] = Math.sin(view); 
		rotMatrix[3][3] = Math.cos(view);
	}


	//-----------------------------------------------------------------------
	//			getWC WorldCoordinates for given surface point; angles a,b
	//_______________________________________________________________________
	
	static public void getWC(double a, double b,double rWC[])  {
		double	r_xz;

		r_xz = ra + rb * Math.cos(b);
		rWC[1] = r_xz * Math.cos(a);
		rWC[2] = rb * Math.sin(b);
		rWC[3] = -r_xz * Math.sin(a);
	}


	//-----------------------------------------------------------------------
	//			getVC ViewCoordinates for given worldCoordinate
	//_______________________________________________________________________
	
	static public void getVC(double rWC[] , double rVC[])  {
		int	i,j;

		for(i=1;i<=3;i++)  {
			rVC[i]=0;
			for(j=1;j<=3;j++)  {
				rVC[i] = rVC[i] + rotMatrix[i][j] * rWC[j];
			}
		}
	}

	//-----------------------------------------------------------------------
	//					initiateData
	//		visibility true, torus_XY_MinMax VCdatabase
	//_______________________________________________________________________

	static void initiateData(int nStepsDa,int nStepsDb)  {
		int		aI,bI;
				
		double	rWC[] = new double[4],
				rVC[] = new double[4];

		torusMarginHor =(int)( (App.graph.curWidth - 1.6*(App.graph.curHeight-2*torusMarginVer)) /2);

		pixMaxLeft = torusMarginHor;
		pixMaxRight = App.graph.curWidth - torusMarginHor;
		pixMaxUp = torusMarginVer;
		pixMaxDown = App.graph.curHeight - torusMarginVer;

		torusXmin = 0;
		torusXmax = 0;
		torusYmin = 0;
		torusXmax = 0;
		for( aI = 0; aI < nStepsDa; aI++ )  {
			for( bI = 0; bI < nStepsDb; bI++ )  {
				polygons[aI][bI] = new PolygonInfo();
				polygons[aI][bI].visible=true;
				getWC(aI*da,bI*db,rWC);
				getVC(rWC,rVC);
				if (rVC[1] < torusXmin) torusXmin=rVC[1];
				if (rVC[1] > torusXmax) torusXmax=rVC[1];
				if (rVC[2] < torusYmin) torusYmin=rVC[2];
				if (rVC[2] > torusYmax) torusYmax=rVC[2];
				vcData[aI][bI][1]=rVC[1];
				vcData[aI][bI][2]=rVC[2];
				vcData[aI][bI][3]=rVC[3];
			}
		}
	}


	//-----------------------------------------------------------------------
	//			getVCfromData  
	//_______________________________________________________________________
	
	static private void getVCfromData(int aI,int bI,double rVC[])  {

		rVC[1]=vcData[aI][bI][1];
		rVC[2]=vcData[aI][bI][2];
		rVC[3]=vcData[aI][bI][3];
	}

	
	//-----------------------------------------------------------------------
	//			vectorSub
	//_______________________________________________________________________
		
	static private void vectorSub(double a[],double b[],double a_b[])  {
		a_b[1]=a[1]-b[1];
		a_b[2]=a[2]-b[2];
		a_b[3]=a[3]-b[3];
	}


	//-----------------------------------------------------------------------
	//			getNormal
	//_______________________________________________________________________
		
	static private void getNormal(double a[],double b[],double N[])  {
		double length;

		N[1]=a[2]*b[3]-a[3]*b[2];
		N[2]=a[3]*b[1]-a[1]*b[3];
		N[3]=a[1]*b[2]-a[2]*b[1];
		length = Math.sqrt(N[1]*N[1]+N[2]*N[2]+N[3]*N[3]);
		N[1] = N[1] / length;
		N[2] = N[2] / length;
		N[3] = N[3] / length;
	}


	//-----------------------------------------------------------------------
	//			getColor
	//_______________________________________________________________________
		
	static private Color getColor(double Nx,double Ny,double Nz,
						 double Lx,double Ly,double Lz)  {
		double	light;
		RGBcol	rgb = new RGBcol(0,0,0);

		light = (light_amb + light_dir * (Nx*Lx+Ny*Ly+Nz*Lz))/light_max;
		rgb = Misc.HLStoRGB( new HLScol(hue,light,sat) );
		return(  new Color( (int)(rgb.red*255), (int)(rgb.green*255), (int)(rgb.blue*255) )  ); 
	}


	//-----------------------------------------------------------------------
	//			setScanInfo
	//_______________________________________________________________________

	static private void setScanInfo(int aI, int bI, 
				double rVC[],double raVC[],double rbVC[],double rabVC[])  {

		Point	pt[] = new Point[4],
				ptSorted[] = new Point[4];

		int		topI,
				i,
				yI,
				pt2,
				nLines,
				scanlineNo;

		double	DxDivDy;
		
		for(i=0;i<4;i++) {
			pt[i] = new Point();
			ptSorted[i] = new Point();
		}

		// points stored in positive anticlockwise direction for all (a,b) 
		// with Nz outwards
		
		pt[0].x = Misc.pix( pixMaxLeft, pixMaxRight, torusXmin, torusXmax, rVC[1]);
		pt[0].y = Misc.pix( pixMaxDown, pixMaxUp, torusYmin, torusYmax, rVC[2]);
		pt[1].x = Misc.pix( pixMaxLeft, pixMaxRight, torusXmin, torusXmax, raVC[1]);
		pt[1].y = Misc.pix( pixMaxDown, pixMaxUp, torusYmin, torusYmax, raVC[2]);
		pt[2].x = Misc.pix( pixMaxLeft, pixMaxRight, torusXmin, torusXmax, rabVC[1]);
		pt[2].y = Misc.pix( pixMaxDown, pixMaxUp, torusYmin, torusYmax, rabVC[2]);
		pt[3].x = Misc.pix( pixMaxLeft, pixMaxRight, torusXmin, torusXmax, rbVC[1]);
		pt[3].y = Misc.pix( pixMaxDown, pixMaxUp, torusYmin, torusYmax, rbVC[2]);

		// sort so that ptSorted[0] is at top with preserved cyclic order
		topI = 0;
		for( i = 0; i < 4 ; i++)  if (pt[i].y < pt[topI].y) topI=i;
		for( i = 0; i < 4 ; i++)  ptSorted[i] = pt[(topI+i) % 4];
		for( i = 0; i < 4 ; i++)  pt[i] = ptSorted[i];

		// fill in LeftEdge and RightEdge pixValues for each scanLine
		nLines = Misc.maxi4Int(pt[0].y,pt[1].y,pt[2].y,pt[3].y) - 
			     Misc.mini4Int(pt[0].y,pt[1].y,pt[2].y,pt[3].y);
		polygons[aI][bI].pixYtop = pt[0].y;
		polygons[aI][bI].nScanlines = nLines;
		polygons[aI][bI].pixLeftEdge = new int[nLines+1];
		polygons[aI][bI].pixRightEdge = new int[nLines+1];

		//left edges
		scanlineNo = 0;
		for (pt2=1; pt2<=3; pt2++) {
			if (pt[pt2].y > pt[pt2-1].y)  {
				nLines = pt[pt2].y - pt[pt2-1].y;
				DxDivDy = (double)(pt[pt2].x - pt[pt2-1].x) / (double)(pt[pt2].y - pt[pt2-1].y);
				for( yI=1; yI<=nLines; yI++ )  {
					polygons[aI][bI].pixLeftEdge[scanlineNo+yI] =
						pt[pt2-1].x + (int)( (yI-1) * DxDivDy );
				}
				scanlineNo = scanlineNo + nLines;
			}
		}
		//right edges
		scanlineNo = 0;
		for  (pt2=3 ; pt2>0 ; pt2-- )  {
			if (pt[pt2].y > pt[ (pt2+1) % 4 ].y)  {
				nLines = pt[pt2].y - pt[ (pt2+1) % 4 ].y;
				DxDivDy = ( (double)(pt[pt2].x-pt[(pt2+1)%4].x)  )  /
						  ( (double)(pt[pt2].y-pt[(pt2+1)%4].y)  );
				for (yI=1 ; yI<=nLines ; yI++ )  {
					polygons[aI][bI].pixRightEdge[scanlineNo + yI] =
						pt[(pt2+1)%4].x + (int)( (yI-1) * DxDivDy );
				}
				scanlineNo = scanlineNo + nLines;
			}
		}
	}

	//-----------------------------------------------------------------------
	//			removeHiddenSurfaces
	//_______________________________________________________________________

	static private void removeHiddenSurfaces()  {
		int		aI,bI,
				a2I,b2I,
				b2Index,
				a2Istart,a2Iend,
				b2Istart,b2Iend;
		boolean	stop;

		b2Istart = 3 * (nStepsDb / 4);
		b2Iend = 5 * (nStepsDb / 4) -1;
		// test polyArea aI,bI for removal
		for (aI=0 ; aI<(nStepsDa/2) ; aI++)  {
			for (bI=(nStepsDb/4)-1 ; bI<3*(nStepsDb/4) ; bI++ )  {
				if ( aI < nStepsDa/4 )  {
					a2Istart = 3 * (nStepsDa/4);
					a2Iend = nStepsDa - 1;
				}
				else  {
					a2Istart = nStepsDa / 2;
					a2Iend = 3 * (nStepsDa / 4) -1;
				}
				a2I = a2Istart;
				stop = false;
				while ( (! stop) && (a2I<=a2Iend) )  {
					if ( polygons[aI][bI].xCenter > segments[a2I].xMin &&		
					     polygons[aI][bI].xCenter < segments[a2I].xMax &&
					     polygons[aI][bI].yCenter > segments[a2I].yMin &&
						 polygons[aI][bI].yCenter < segments[a2I].yMax	 )  {
						b2I = b2Istart;
						while ( (! stop) && (b2I<=b2Iend) )  {
							b2Index = b2I % nStepsDb ;
							if ( polygons[aI][bI].xCenter > polygons[a2I][b2Index].xMin &&
								 polygons[aI][bI].xCenter < polygons[a2I][b2Index].xMax &&
								 polygons[aI][bI].yCenter > polygons[a2I][b2Index].yMin &&
								 polygons[aI][bI].yCenter < polygons[a2I][b2Index].yMax &&
								 // zAxis towatds viewer
								 polygons[a2I][b2Index].zCenter > polygons[aI][bI].zCenter )  {
								
								polygons[aI][bI].visible=false;
								stop = true;
							}
							b2I++;
						}
					}
					a2I++;
				}
			}
		}
	}


	//-----------------------------------------------------------------------
	//			setTorusData
	//_______________________________________________________________________

	static public void setTorusData()  {

		int		aI = 0,
				bI = 0;

		double	rVC[] = new double[4],
				raVC[] = new double[4],
				rbVC[] = new double[4],
				rabVC[] = new double[4],
				rdaVC[] = new double[4],	// incr. vector along big circle (arm1)
				rdbVC[] = new double[4],	// incr. vector along small circle (arm2)
				N[] = new double[4];

		
		rgb = new RGBcol( (double)(torusColor.getRed()) /255,
						  (double)(torusColor.getGreen()) /255,
						  (double)(torusColor.getBlue()) /255      );
		hls = Misc.RGBtoHLS(rgb);
		hue = hls.hue;
		light = hls.light;
		sat = hls.sat;

		// make database with ViewCoords for (a,b) points
		nStepsDa = (int) ((2*Math.PI) / da );
		nStepsDb = (int) ((2*Math.PI) / db );
		segments = new SegmentInfo[nStepsDa];
		vcData = new double[nStepsDa][nStepsDb][4];

		polygons = new PolygonInfo[nStepsDa][nStepsDb];
		getRotMatrix();
		initiateData(nStepsDa,nStepsDb);

		// put info in segments and polygons 
		for( aI=0 ; aI<nStepsDa ; aI++ )  {
			segments[aI] = new SegmentInfo();
			segments[aI].xMin = 1000;
			segments[aI].xMax = -1000;
			segments[aI].yMin = 1000;
			segments[aI].yMax = -1000;
			for( bI=0 ; bI<nStepsDb ; bI++ )  {
				getVCfromData( aI, bI, rVC);
				getVCfromData( (aI+1) % nStepsDa, bI, raVC);
				getVCfromData( aI, (bI+1) % nStepsDb, rbVC);
				getVCfromData( (aI+1) % nStepsDa, (bI+1) % nStepsDb, rabVC);
				polygons[aI][bI].xCenter = (rVC[1]+raVC[1]+rbVC[1]+rabVC[1])/4;
				polygons[aI][bI].yCenter = (rVC[2]+raVC[2]+rbVC[2]+rabVC[2])/4;
				polygons[aI][bI].zCenter = (rVC[3]+raVC[3]+rbVC[3]+rabVC[3])/4;
				vectorSub(raVC,rVC,rdaVC);
				vectorSub(rbVC,rVC,rdbVC);
				getNormal(rdaVC,rdbVC,N);
				if ( N[3] <= 0 )  {
					polygons[aI][bI].visible = false;
				} 
				else  {
					polygons[aI][bI].Nx=N[1];
					polygons[aI][bI].Ny=N[2];
					polygons[aI][bI].Nz=N[3];
					polygons[aI][bI].color = getColor(N[1],N[2],N[3],light_x,light_y,light_z);
					setScanInfo(aI,bI,rVC,raVC,rbVC,rabVC);
					// update MinMax for segment
					polygons[aI][bI].xMin = Misc.mini4(rVC[1],raVC[1],rbVC[1],rabVC[1]);
					polygons[aI][bI].xMax = Misc.maxi4(rVC[1],raVC[1],rbVC[1],rabVC[1]);
					polygons[aI][bI].yMin = Misc.mini4(rVC[2],raVC[2],rbVC[2],rabVC[2]);
					polygons[aI][bI].yMax = Misc.maxi4(rVC[2],raVC[2],rbVC[2],rabVC[2]);
					if (polygons[aI][bI].xMin < segments[aI].xMin) 
						segments[aI].xMin = polygons[aI][bI].xMin;
					if (polygons[aI][bI].xMax > segments[aI].xMax) 
						segments[aI].xMax = polygons[aI][bI].xMax;
					if (polygons[aI][bI].yMin < segments[aI].yMin) 
						segments[aI].yMin = polygons[aI][bI].yMin;
					if (polygons[aI][bI].yMax > segments[aI].yMax) 
						segments[aI].yMax = polygons[aI][bI].yMax;
				}
			}
		}
		removeHiddenSurfaces();
  }



	//-----------------------------------------------------------------------
	//							circles
	//_______________________________________________________________________
	static public void circles(Graphics g) {
		// angular separation of circles on surface 
		// circle A big arm1
		// circle B small arm2
		int		aI,bI,
				ptX,ptY,
				ptXprev,ptYprev;

		double	deltaA = (2*Math.PI)/20,
				deltaB = (2*Math.PI)/12,
				aRad,bRad,
				rWC[] =new double[4],
				rVC[]= new double[4];
				
		
		g.setColor(Color.darkGray);
		for ( bRad=0; bRad<=2*Math.PI; bRad += deltaB)  {
			aRad = 0;
			getWC( aRad, bRad, rWC );
			getVC( rWC, rVC );
			ptXprev = Misc.pix( pixMaxLeft, pixMaxRight, torusXmin, torusXmax, rVC[1]);
			ptYprev = Misc.pix( pixMaxDown, pixMaxUp, torusYmin, torusYmax, rVC[2]);
			for ( aRad=da; aRad< 2*Math.PI + da; aRad += da)  {
				getWC(aRad, bRad, rWC);
				getVC(rWC, rVC);
				ptX = Misc.pix( pixMaxLeft, pixMaxRight, torusXmin, torusXmax, rVC[1]);
				ptY = Misc.pix( pixMaxDown, pixMaxUp, torusYmin, torusYmax, rVC[2]);
				aI = (int) (aRad / da) % nStepsDa;
				bI = (int) (bRad / db) % nStepsDb;
				if (polygons[aI][bI].visible)
					g.drawLine(ptXprev,ptYprev,ptX,ptY);
				ptXprev = ptX;
				ptYprev = ptY;
			}
		}		
		for ( aRad=0; aRad<=2*Math.PI; aRad += deltaA)  {
			bRad = 0;
			getWC( aRad, bRad, rWC );
			getVC( rWC, rVC );
			ptXprev = Misc.pix( pixMaxLeft, pixMaxRight, torusXmin, torusXmax, rVC[1]);
			ptYprev = Misc.pix( pixMaxDown, pixMaxUp, torusYmin, torusYmax, rVC[2]);
			for ( bRad=db; bRad< 2*Math.PI + db; bRad += db)  {
				getWC(aRad, bRad, rWC);
				getVC(rWC, rVC);
				ptX = Misc.pix( pixMaxLeft, pixMaxRight, torusXmin, torusXmax, rVC[1]);
				ptY = Misc.pix( pixMaxDown, pixMaxUp, torusYmin, torusYmax, rVC[2]);
				aI = (int) (aRad / da) % nStepsDa;
				bI = (int) (bRad / db) % nStepsDb;
				if (polygons[aI][bI].visible)
					g.drawLine(ptXprev,ptYprev,ptX,ptY);
				ptXprev = ptX;
				ptYprev = ptY;
			}
		}
		
	}

	//-----------------------------------------------------------------------
	//							axes
	//_______________________________________________________________________	
	static public void axes(Graphics g)  {
		int		lineW = 2,
				lineVer,
				lineHor,
				ptX,ptY,
				xPts[] = new int[5],
				yPts[] = new int[5];

		double	a0,b0,
				rWC[] = new double[4],
				rVC[] = new double[4];
	
		lineVer = App.graph.curHeight/10;
		lineHor = App.graph.curWidth/20;
		a0 = -Math.PI / 2;
		b0 = 0;		
		getWC( a0, b0, rWC );
		getVC( rWC, rVC );
		ptX = Misc.pix( pixMaxLeft, pixMaxRight, torusXmin, torusXmax, rVC[1]);
		ptY = Misc.pix( pixMaxDown, pixMaxUp, torusYmin, torusYmax, rVC[2]);
		g.setColor(Color.darkGray);		

		//axis a hor., Arm1 
		xPts[0] = ptX;				yPts[0] = ptY + lineW;
		xPts[1] = ptX + lineHor;	yPts[1] = ptY + lineW;
		xPts[2] = ptX + lineHor;	yPts[2] = ptY - lineW;
		xPts[3] = ptX;				yPts[3] = ptY - lineW;
		xPts[4] = xPts[0];			yPts[4] = yPts[0];
		g.fillPolygon(xPts,yPts,5);			
		g.drawString("arm1",ptX + lineHor +3,ptY + 10);

		//axis b ver., Arm2 
		xPts[0] = ptX - lineW;	yPts[0] = ptY;
		xPts[1] = ptX + lineW;	yPts[1] = ptY;
		xPts[2] = ptX + lineW;	yPts[2] = ptY - lineVer;
		xPts[3] = ptX - lineW;	yPts[3] = ptY - lineVer;
		xPts[4] = xPts[0];		yPts[4] = yPts[0];
		g.fillPolygon(xPts,yPts,5);			
		g.drawString("arm2",ptX - 40, ptY - 10);
	}

	//-----------------------------------------------------------------------
	//							dot
	//_______________________________________________________________________	
	static public void dot(Graphics g)  {
		boolean abs;

		int		ptX,ptY,
				aI,bI;

		double	aRad,bRad,				
				phi1,phi2,
				rWC[] = new double[4],
				rVC[] = new double[4];

		abs = Graph.arm2Abs;
		phi1 = Graph.phi1_start[Graph.origin];
		phi2 = Graph.phi2_start[Graph.origin];
		aRad = phi1;
		bRad = abs ? (phi1 + phi2 + Math.PI/2 ) : phi2;
		aI = (int)(aRad/da) % nStepsDa ;
		if (aI<0) aI += nStepsDa;
		bI = (int)(bRad/db) % nStepsDb ;
		if (bI<0) bI += nStepsDb;
		getWC( aRad, bRad, rWC );
		getVC( rWC, rVC );
		ptX = Misc.pix( pixMaxLeft, pixMaxRight, torusXmin, torusXmax, rVC[1]);
		ptY = Misc.pix( pixMaxDown, pixMaxUp, torusYmin, torusYmax, rVC[2]);
		g.setColor(Color.black);
		if (polygons[aI][bI].visible)  
			g.fillOval(ptX-5,ptY-5,10,10);
		else
			g.drawOval(ptX-5,ptY-5,10,10);
	}
	


	//-----------------------------------------------------------------------
	//								drawTorus
	//_______________________________________________________________________
	static public void drawTorus(Graphics g)  {
		int		aI,bI,
				yI,yTop;

		Font	font = new Font("Helvetica", Font.BOLD, App.fontSize + 6 );

		String  headline = "Angular positions of arms";

		g.clearRect(0,0,App.graph.curWidth,App.graph.curHeight);
		g.setFont(font);
		g.drawString(headline, (Graph.curWidth - g.getFontMetrics().stringWidth(headline)) / 2, 20);
		g.setFont( App.regFont );
		for( aI=0; aI<nStepsDa; aI++)  {
			for( bI=0; bI<nStepsDb; bI++)	{
				if (polygons[aI][bI].visible)  {
					g.setColor( polygons[aI][bI].color );
					yTop = polygons[aI][bI].pixYtop;
					for (yI=1; yI<=polygons[aI][bI].nScanlines; yI++)  {
						g.drawLine( polygons[aI][bI].pixLeftEdge[yI], yTop + yI,
									polygons[aI][bI].pixRightEdge[yI], yTop + yI);
					}
				}
			}
		}
	}

}

