Attribute VB_Name = "modCol"
'********************************************************************************
'   RGB-modell Rtt,grnt och btt angivna i styrka 0->1
'********************************************************************************
'   HLS-modell dubbelkonad frgrymd med ljusstyka i Vertikal led
'   H Hue motsvarar kulr angiven som vinkel i cirkulrt spektra
'       Rd=0 Gul=60 Grn=120 Cyan=180 Bl=240 Magenta=300 Rd=360
'   L Lightness ljusstyrka gr frn svart=0 till vitt=1 rena frger=0.5
'   S Saturation frgmttnad 0->1 0 grskala=0  ren frg=1
'********************************************************************************
'   NCS-modell LLSS-CnnC t.ex. 3080-R20B dubbelkonad frgrymd med ljustyrka
'   LL ljusstyrka  svart 0->100 vitt
'   SS mttnad     grskala 0->100 ren frg
'   CnnC motsvarar kulr i cirkulrt spektra med fyra grundfrger i vinklarna
'   Rd=0 Gul=90 Grn=180 Bl=270
'   mellanliggande frger anges med nn 01-99
'   rknat medurs frn Rd->Bl->Grn->Gul
'********************************************************************************

Const hlsRed As Integer = 0
Const hlsYellow As Integer = 60
Const hlsGreen As Integer = 120
Const hlsCyan As Integer = 180
Const hlsBlue As Integer = 240
Const hlsMagenta As Integer = 300

Const ncsRed As Single = 0
Const ncsYellow As Single = 90
Const ncsGreen As Single = 180
Const ncsBlue As Single = 270



' ********************************************************************
'   Konvertering frn RGB-frgmodell till HLS-frgmodell
' ********************************************************************
Public Sub RGBtoHLS(r As Single, g As Single, b As Single, _
                    H As Single, L As Single, S As Single)
                    
Const small As Single = 0.0000001
Dim maxVal As Single
Dim minVal As Single
Dim diff As Single
Dim rDist As Single
Dim gDist As Single
Dim bDist As Single

maxVal = Max(r, g, b)
minVal = Min(r, g, b)
diff = maxVal - minVal
L = (maxVal + minVal) / 2
If Abs(diff) < small Then
    S = 0
    H = 0 ' om R=G=B r egentligen H (kulr) odefinierad
Else
    If L <= 0.5 Then
        S = diff / (maxVal + minVal)
    Else
        S = diff / (2 - maxVal - minVal)
    End If
    rDist = (maxVal - r) / diff
    gDist = (maxVal - g) / diff
    bDist = (maxVal - b) / diff
    If r = maxVal Then
        H = bDist - gDist
    ElseIf g = maxVal Then
        H = 2 + rDist - bDist
    ElseIf b = maxVal Then
        H = 4 + gDist - rDist
    End If
    H = H * 60
    If H < 0 Then
        H = H + 360
    End If
End If
End Sub   ' RGBtoHLS



' ********************************************************************
'   Konvertering frn HLS-frgmodell till RGB-frgmodell
' ********************************************************************
Public Sub HLStoRGB(H As Single, L As Single, S As Single, _
                    r As Single, g As Single, b As Single)
Dim p1 As Single
Dim p2 As Single

If L < 0.5 Then
    p2 = L * (1 + S)
Else
    p2 = L + S - L * S
End If
p1 = 2 * L - p2
If S = 0 Then
    r = L
    g = L
    b = L
Else
    r = RGB(p1, p2, H + 120)
    g = RGB(p1, p2, H)
    b = RGB(p1, p2, H - 120)
End If
End Sub  ' HLS to RGB

Private Function RGB(q1 As Single, q2 As Single, hue As Single) As Single
If hue > 360 Then hue = hue - 360
If hue < 0 Then hue = hue + 360
If hue < 60 Then
    RGB = q1 + (q2 - q1) * hue / 60
ElseIf hue < 180 Then
    RGB = q2
ElseIf hue < 240 Then
    RGB = q1 + (q2 - q1) * (240 - hue) / 60
Else
    RGB = q1
End If
End Function



Private Function Max(a As Single, b As Single, c As Single) As Single

If a > b Then
    Max = a
Else
    Max = b
End If
If c > Max Then
    Max = c
End If
End Function

Private Function Min(a As Single, b As Single, c As Single) As Single

If a < b Then
    Min = a
Else
    Min = b
End If
If c < Min Then
    Min = c
End If
End Function


'********************************************************************************
'   NCS-modell LLSS-CnnC t.ex. 3080-R20B dubbelkonad frgrymd med ljustyrka
'   LL ljusstyrka  svart 0->100 vitt
'   SS mttnad     grskala 0->100 ren frg
'   CnnC motsvarar kulr i cirkulrt spektra med fyra grundfrger i vinklarna
'   Rd=0 Gul=90 Grn=180 Bl=270
'   mellanliggande frger anges med nn 01-99
'   rknat medurs frn Rd->Bl->Grn->Gul
'********************************************************************************


' ********************************************************************
'   Konvertering frn NCS-frgmodell till HLS-frgmodell
' ********************************************************************
Public Sub NCStoHLS(NCS As String, H As Single, L As Single, S As Single)
Dim c1 As String, c2 As String
Dim LL As String, SS As String
Dim nn As Integer

LL = Val(Left(NCS, 2))
SS = Val(Mid(NCS, 3, 2))


L = (100 - LL) / 100 - 0.5 * SS / 100
If L = 1 Or L = 0 Then
    S = 0
Else
    If L > 0.5 Then
        S = 0.5 * SS / (100 * (1 - L))
    Else
        S = 0.5 * SS / (100 * L)
    End If
End If

If Len(NCS) = 4 Then ' grskala
    H = 0 ' egentligen odefinierat
End If
If Len(NCS) = 6 Then
    If Right(NCS, 1) = "R" Then H = hlsRed
    If Right(NCS, 1) = "Y" Then H = hlsYellow
    If Right(NCS, 1) = "G" Then H = hlsGreen
    If Right(NCS, 1) = "B" Then H = hlsBlue
Else
    c1 = UCase(Mid(NCS, 6, 1))
    nn = Val(Mid(NCS, 7, 2))
    c2 = UCase(Mid(NCS, 9, 1))
    If c1 = "R" And c2 = "B" Then
        H = hlsBlue + ((100 - nn) / 100) * (360 - hlsBlue) ' hlsRed =0 not 360
    End If
    If c1 = "B" And c2 = "G" Then
        H = hlsGreen + ((100 - nn) / 100) * (hlsBlue - hlsGreen)
    End If
    If c1 = "G" And c2 = "Y" Then
        H = hlsYellow + ((100 - nn) / 100) * (hlsGreen - hlsYellow)
    End If
    If c1 = "Y" And c2 = "R" Then
        H = hlsRed + ((100 - nn) / 100) * (hlsYellow - hlsRed)
    End If
End If
End Sub


' ********************************************************************
'   Konvertering frn HLS-frgmodell till NCS-frgmodell
' ********************************************************************
Public Sub HLStoNCS(H As Single, L As Single, S As Single, NCS As String)
Dim c1 As String, c2 As String
Dim LL0 As Single
Dim LL As Single, SS As Single
Dim nn As Integer


LL0 = 100 * (1 - L)
If L > 0.5 Then
    SS = S / 1 * (2 * LL0)
Else
    SS = S / 1 * (2 * (100 - LL0))
End If
LL = LL0 - 50 * SS / 100
    
If Int(LL) = 100 Then
    NCS = "99"
Else
    NCS = Format(Int(LL), "00")
End If

If Int(SS) = 0 Then   ' grskala
    NCS = NCS + "00"
    Exit Sub
ElseIf Int(SS) = 100 Then
    NCS = NCS + "99"
Else
    NCS = NCS + Format(Int(SS), "00")
End If
    

If H = hlsRed Then NCS = NCS + "-R"     '0
If H = hlsYellow Then NCS = NCS + "-Y"  '60
If H = hlsGreen Then NCS = NCS + "-G"   '120
If H = hlsBlue Then NCS = NCS + "-B"    '240

If H > hlsRed And H < hlsYellow Then
    NCS = NCS + "-Y" + Format(Int(100 * (hlsYellow - H) / 60), "00") + "R"
    
ElseIf H > hlsYellow And H < hlsGreen Then
    NCS = NCS + "-G" + Format(Int(100 * (hlsGreen - H) / 60), "00") + "Y"
    
ElseIf H > hlsGreen And H < hlsBlue Then
    NCS = NCS + "-B" + Format(Int(100 * (hlsBlue - H) / 120), "00") + "G"
    
ElseIf H > hlsBlue And H < 360 Then   ' hlsRed=0 not 360
    NCS = NCS + "-R" + Format(Int(100 * (360 - H) / 120), "00") + "B"
End If
End Sub

